@@ -35,7 +35,6 @@ DEFAULT_HOST = "api.mpcontribs.org"
 BULMA = "is-narrow is-fullwidth has-background-light"
 
 j2h = Json2Html()
-quantity_keys = {"display", "value", "unit"}
 pd.options.plotting.backend = "plotly"
 warnings.formatwarning = lambda msg, *args, **kwargs: f"{msg}\n"
 warnings.filterwarnings("default", category=DeprecationWarning, module=__name__)
@@ -94,7 +93,7 @@ class FidoClientGlobalHeaders(FidoClient):
 
 
 def visit(path, key, value):
-    if isinstance(value, dict) and quantity_keys == set(value.keys()):
+    if isinstance(value, dict) and "display" in value:
         return key, value["display"]
     return True
 
@@ -293,9 +292,17 @@ class Client(SwaggerClient):
                     ]
 
                     for future in as_completed(futures):
-                        resp = future.result().json()
-                        cids += [d["id"] for d in resp["data"]]
-                        pbar.update(len(resp["data"]))
+                        response = future.result()
+                        status = response.status_code
+                        if status in [200, 400, 401, 404]:
+                            resp = response.json()
+                            if status == 200:
+                                cids += [d["id"] for d in resp["data"]]
+                                pbar.update(len(resp["data"]))
+                            else:
+                                warnings.warn(resp["error"])
+                        else:
+                            warnings.warn(response.content.decode("utf-8"))
 
                     pbar.refresh()
                     pbar.set_description("Delete contribution(s)")
@@ -314,11 +321,16 @@ class Client(SwaggerClient):
 
                     resp = None
                     for future in as_completed(futures):
-                        resp = future.result().json()
-                        if "count" in resp:
-                            pbar.update(resp["count"])
-                        elif "error" in resp:
-                            warnings.warn(resp["error"])
+                        response = future.result()
+                        status = response.status_code
+                        if status in [200, 400, 401, 404]:
+                            resp = response.json()
+                            if status == 200:
+                                pbar.update(resp["count"])
+                            else:
+                                warnings.warn(resp["error"])
+                        else:
+                            warnings.warn(response.content.decode("utf-8"))
 
                     if resp and resp.get("count"):
                         self.load()
@@ -368,17 +380,24 @@ class Client(SwaggerClient):
                     ]
 
                     for future in as_completed(futures):
-                        resp = future.result().json()
+                        response = future.result()
+                        status = response.status_code
+                        if status in [200, 400, 401, 404]:
+                            resp = response.json()
+                            if status == 200:
+                                for contrib in resp["data"]:
+                                    existing["ids"].add(contrib["id"])
+                                    existing["identifiers"].add(contrib["identifier"])
 
-                        for contrib in resp["data"]:
-                            existing["ids"].add(contrib["id"])
-                            existing["identifiers"].add(contrib["identifier"])
+                                    for component in ["structures", "tables"]:
+                                        md5s = set(d["md5"] for d in contrib[component])
+                                        existing[component] |= md5s
 
-                            for component in ["structures", "tables"]:
-                                md5s = set(d["md5"] for d in contrib[component])
-                                existing[component] |= md5s
-
-                            pbar.update(1)
+                                    pbar.update(1)
+                            else:
+                                warnings.warn(resp["error"])
+                        else:
+                            warnings.warn(response.content.decode("utf-8"))
 
                 nexisting = 0
                 if unique_identifiers and existing["identifiers"]:
@@ -479,11 +498,16 @@ class Client(SwaggerClient):
                     ]
 
                     for future in as_completed(futures):
-                        resp = future.result().json()
-                        if "count" in resp:
-                            pbar.update(resp["count"])
-                        elif "error" in resp:
-                            warnings.warn(resp["error"])
+                        response = future.result()
+                        status = response.status_code
+                        if status in [201, 400, 401, 404]:
+                            resp = response.json()
+                            if status == 201:
+                                pbar.update(resp["count"])
+                            else:
+                                warnings.warn(resp["error"])
+                        else:
+                            warnings.warn(response.content.decode("utf-8"))
 
                     if resp and resp.get("count"):
                         self.load()
